/*
 * Copyright (C) 1996 Michael Davidson.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#pragma ident	"@(#)filesys.c	1.5	99/02/18 SMI"

#include	<errno.h>
#include	<sys/types.h>
#include	<sys/statfs.h>
#include	<string.h>
#include	"lxrun.h"

/*
 * filesystem specific system calls
 */

/*
 *	mount		(NOT IMPLEMENTED)
 *	umount		(NOT IMPLEMENTED)
 *	statfs
 *	fstatfs
 *	setfsuid	(NOT IMPLEMENTED)
 *	setfsgid	(NOT IMPLEMENTED)
 */

int lx_mount()		{ return enosys("mount"); }
int lx_umount()		{ return enosys("umount"); }

typedef struct {
	int __val[2];
} fsid_t;

struct lx_statfs {
	 long    f_type;     /* type of filesystem (see below) */
	 long    f_bsize;    /* optimal transfer block size */
	 long    f_blocks;   /* total data blocks in file system */
	 long    f_bfree;    /* free blocks in fs */
	 long    f_bavail;   /* free blocks avail to non-superuser */
	 long    f_files;    /* total file nodes in file system */
	 long    f_ffree;    /* free file nodes in fs */
	 fsid_t  f_fsid;     /* file system id */
	 long    f_namelen;  /* maximum length of filenames */
	 long    f_spare[6]; /* spare for later */
};

static int
copy_out_new_statfs(struct statfs *from, struct lx_statfs *to)
{
	if (! verify_write(to, sizeof(*to)))
		return -1;

	(void) memset((void *)to, 0, sizeof(*to));

	to->f_type =	from->f_fstyp;
	to->f_bsize =	from->f_bsize;
	to->f_blocks =	from->f_blocks;
	to->f_bfree =	from->f_bfree;

	to->f_files =	from->f_files;
	to->f_ffree =	from->f_ffree;

	/*
	 * Fields that are undefined for a particular file system are set to -1.
	 * (see statfs(2)).
	 */
	to->f_namelen = -1;

	return 0;
}

int
lx_statfs(const char *path, struct lx_statfs *buf)
{
	int ret;
	struct statfs nbuf;

	if ((ret = statfs(path, &nbuf, sizeof(struct statfs), 0)) == 0)
		ret = copy_out_new_statfs(&nbuf, buf);

	return ret;
}

int
lx_fstatfs(int fd, struct lx_statfs *buf)
{
	int ret;
	struct statfs nbuf;

	if ((ret = fstatfs(fd, &nbuf, sizeof(struct statfs), 0)) == 0)
		ret = copy_out_new_statfs(&nbuf, buf);

	return ret;
}

int lx_setfsuid()	{ return enosys("setfsuid"); }
int lx_setfsgid()	{ return enosys("setfsgid"); }
