/*
 * Copyright (C) 1998 Steve Ginzburg.
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose and without fee is hereby
 * granted, provided that the above copyright notice appear in all
 * copies and that both that copyright notice and this permission
 * notice appear in supporting documentation.
 *
 * This software is provided "as is" without express or implied warranty.
 */

#pragma ident	"@(#)procfs.c	1.2	99/02/18 SMI"

#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


/*
** Write something to "out" that looks like what we'd expect to get
** by cat(1)ing "name" on a Linux system.
**
** Right now, we emulate a lot of /proc stuff, but most of the stuff
** we print out are static dummy values.  If we encounter a Linux
** program that needs real information, we should update this routine.
*/
void emulation(FILE *out, char *name)
{
	char *pid = getenv("LINUX_PID");

	if(strncmp(name, "/proc/", 6) != 0)
		return;
	name+=6;

	if((strncmp(name, pid, strlen(pid)) && name[strlen(pid)]=='/') ||
	    strncmp(name, "self/", 5)==0)
		{
		name=strchr(name, '/')+1;
		if(strcmp(name, "cmdline")==0)
			fprintf(out, "%s", getenv("LINUX_CMDLINE"));
		else if(strcmp(name, "cwd")==0)
			fprintf(out, "%s", getcwd(NULL, 1024));
		}
	else if(strcmp(name, "cmdline")==0)
		{
		/* Hard-wired dummy value */
		fprintf(out, "auto BOOT_IMAGE=linux ro root=302");
		}
	else if(strcmp(name, "cpuinfo")==0)
		{
		/* These are hard-wired dummy values right now. */
		fprintf(out,   "processor	: 0\n");
		fprintf(out,   "cpu		: 6x86\n");
		fprintf(out,   "model		: 166\n");
		fprintf(out,   "vendor_id	: CyrixInstead\n");
		fprintf(out,   "stepping	: 4\n");
		fprintf(out,   "fdiv_bug	: no\n");
		fprintf(out,   "hlt_bug		: no\n");
		fprintf(out,   "fpu		: yes\n");
		fprintf(out,   "fpu_exception	: yes\n");
		fprintf(out,   "cpuid		: yes\n");
		fprintf(out,   "wp		: yes\n");
		fprintf(out,   "flags		: fpu\n");
		fprintf(out,   "bogomips	: 3.14159\n");
		}
	else if(strcmp(name, "filesystems")==0)
		{
		fprintf(out,   "	htfs\n");
		fprintf(out,   "	dtfs\n");
		fprintf(out,   "	s51k\n");
		fprintf(out,   "	msdos\n");
		fprintf(out,   "nodev	proc\n");
		fprintf(out,   "nodev	nfs\n");
		fprintf(out,   "	iso9660\n");
		}
	else if(strcmp(name, "loadavg")==0)
		{
		fprintf(out,   "0.00 0.00 0.00 0/1 123");
		}
	else if(strcmp(name, "mounts")==0)
		{
		fprintf(out,   "rootfs / htfs rw 0 0\n");
		fprintf(out,   "none /proc proc rw 0 0\n");
		}
	else if(strcmp(name, "uptime")==0)
		{
		fprintf(out,   "1000.00 500.00");
		}
	else if(strcmp(name, "version")==0)
		{
		fprintf(out,   "Linux version 2.666.999 (lxrun@kicks.ass) (gcc version 2.7.2) #1 %s", __DATE__);
		}
}


int main(int argc, char *argv[])
{
	int fd;		/* fd to which we must write our output */
	FILE *out;

	if(argc != 3)
		{
		write(2, "This program is intended to be invoked by lxrun.\n",
			49);
		exit(1);
		}

	fd=atoi(argv[1]);

	if((out=fdopen(fd, "w")) == NULL)
		{
		fprintf(stderr, "%s: Can't fdopen %d\n", argv[0], fd);
		exit(2);
		}

	emulation(out, argv[2]);

	fclose(out);
}
