
/* tioclinux.c */

/*
** Emulation of TIOCLINUX ioctl (some subcodes) for SCO OpenServer 5.0.x
**
** Copyright (c) 1998 Steven Ginzburg, all rights reserved.
**
** Sep  6 1998	- Initial revision
**
*/

#pragma ident	"@(#)tioclinux.c	1.2	99/02/18 SMI"

#include <sys/console.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>

extern int errno;

char sel_buffer[256];
int sel_count=0;

typedef struct region {
	short xs, ys, xe, ye;	/* Selection boundaries, 0-based */
	short sel_mode;		/* 0 = char, 1 = word, 2 = line */
} Region;


Region hilited;


void toggle_highlight(Region *r)
{
	int start, end, i;
	char *disp;			/* display memory */

	disp=(char *)ioctl(0, MAPCONS, 0L);

	if(r->sel_mode == 1)		/* word-based selection */
		{
		/* Not implemented.  Default to char-based. */
		}
	else if(r->sel_mode == 2)	/* line-based selection */
		{
		r->xs = 0; r->xe = 79;
		}

	start = r->xs*2 + 80*r->ys*2;
	end = r->xe*2 + 80*r->ye*2;

	/* Guarantee start <= end */
	if(start > end)
		start ^= end ^= start ^= end;

	sel_count = end-start;
	if(sel_count > sizeof sel_buffer)
		sel_count = sizeof sel_buffer;

	for(i=0; i<=sel_count && i<sizeof sel_buffer; ++i)
		{
		disp[i+start+1] ^= 0x50;
		sel_buffer[i] = disp[i+start];
		}
}


int copy(Region *r)
{
	/* unhighlight previous region */
	if(sel_count > 0)
		toggle_highlight(&hilited);

	memcpy(&hilited, r, sizeof(Region));

	toggle_highlight(&hilited);

	return 0;
}


int tioclinux(int fd, void *arg)
{
	switch(*((char *)arg))
		{
		case 2:
			{
			Region r;
			memcpy(&r, ((char *)arg)+1, sizeof(Region));
			--r.xs; --r.ys; --r.xe; --r.ye;
			return copy(&r);
			}
		case 3:
			return write(fd, sel_buffer, sel_count);
		default:
			errno=ENOSYS; return -1;
		}
}
